"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_session_1 = __importDefault(require("express-session"));
const passport_1 = __importDefault(require("passport"));
const passport_google_oauth20_1 = require("passport-google-oauth20");
const routes_1 = __importDefault(require("../routes"));
const utils_1 = require("./utils");
const models_1 = require("../models");
const branch_resolver_1 = __importDefault(require("../middleware/branch-resolver"));
const user_resolver_1 = __importDefault(require("../middleware/user-resolver"));
const bodyParser = require("body-parser");
const cookieParser = require("cookie-parser");
const express = require("express");
const http = require("http");
const cors = require("cors");
// eslint-disable-next-line @typescript-eslint/no-var-requires, @typescript-eslint/naming-convention
const KnexSessionStore = require('connect-session-knex')(express_session_1.default);
const uuid_1 = require("uuid");
const devMode = Boolean(process.env.DEV_MODE);
const appHost = process.env.APP_HOST || 'http://dev.scpdb.org';
const sessionSecret = process.env.SESSION_SECRET;
if (!sessionSecret) {
    throw new Error('Session secret is required');
}
const googleClientId = process.env.GOOGLE_CLIENT_ID;
const googleClientSecret = process.env.GOOGLE_CLIENT_SECRET;
if (!googleClientId || !googleClientSecret) {
    throw new Error('Google client id and secret are required');
}
passport_1.default.use(new passport_google_oauth20_1.Strategy({
    clientID: googleClientId,
    clientSecret: googleClientSecret,
    callbackURL: `${appHost}/auth/google/callback`,
}, (accessToken, refreshToken, profile, cb) => {
    models_1.User.findOrCreateByGoogleId(profile.id)
        .then((id) => {
        return cb(undefined, { id });
    })
        .catch((error) => {
        return cb(error, undefined);
    });
}));
passport_1.default.serializeUser((user, done) => {
    done(null, user);
});
passport_1.default.deserializeUser((user, done) => {
    done(null, user);
});
const CORS_WHITELIST = [
    'https://m.scpfoundation.net',
    'http://dev.scpdb.org',
    'https://scpdev.resure.net',
];
function stringifyHeaders(headers) {
    if (!process.env.DEV_LOG_HEADERS) {
        return '';
    }
    return '-- ' + Object.keys(headers).map((name) => {
        return `${name}: ${headers[name]}`;
    }).join(' • ');
}
// eslint-disable-next-line @typescript-eslint/no-var-requires
const httpLogger = require('pino-http')({
    logger: utils_1.logger,
    genReqId: uuid_1.v4,
    serializers: {
        req: (req) => {
            if (devMode) {
                return `${req.method} ${req.url} ${stringifyHeaders(req.headers)}`.trim();
            }
            else {
                return {
                    id: req.id,
                    method: req.method,
                    url: req.url,
                    headers: req.headers,
                };
            }
        },
        res: (res) => {
            if (devMode) {
                return res.statusCode;
            }
            else {
                return { statusCode: res.statusCode };
            }
        },
    },
});
function generateServer() {
    const app = express();
    const router = express.Router(); // eslint-disable-line new-cap
    app.disable('x-powered-by');
    app.use((req, res, next) => {
        req.devMode = devMode;
        next();
    });
    app.use(httpLogger);
    app.use(utils_1.sentry.requestHandler());
    app.use(bodyParser.json());
    app.use(cors({
        origin: CORS_WHITELIST,
    }));
    app.use(cookieParser());
    app.use(express_session_1.default({
        name: 'scp_session',
        cookie: {
            maxAge: 1000 * 60 * 60 * 24 * 30 * 2,
        },
        secret: sessionSecret,
        store: new KnexSessionStore({ knex: models_1.db }),
        resave: false,
        saveUninitialized: false,
    }));
    app.use(passport_1.default.initialize());
    app.use(passport_1.default.session());
    app.use(branch_resolver_1.default);
    app.use(user_resolver_1.default);
    app.get('/auth/google', (req, res) => {
        const returnPath = req.query.returnPath;
        if (returnPath && returnPath.startsWith('/') && req.session) {
            req.session.returnPath = returnPath;
        }
        passport_1.default.authenticate('google', { scope: ['profile'] })(req, res);
    });
    app.get('/auth/google/callback', passport_1.default.authenticate('google', { failureRedirect: '/' }), (req, res) => {
        var _a;
        res.redirect(((_a = req.session) === null || _a === void 0 ? void 0 : _a.returnPath) || '/');
    });
    app.get('/auth/logout', (req, res) => {
        req.logout();
        res.redirect('/');
    });
    app.get('/auth/whoami', (req, res) => {
        if (req.userInfo) {
            const { id, username, wikidotUid } = req.userInfo;
            res.send({ id, username, wikidotUid });
        }
        else {
            res.send({ id: null });
        }
    });
    routes_1.default(app);
    app.use('/', router);
    app.use(utils_1.sentry.errorHandler());
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    app.use((err, req, res, next) => {
        if (req.log) {
            req.log.error(err, 'Unhandled error');
        }
        else {
            utils_1.reportError(err, { message: 'Unhandled error' });
        }
        res.status(500).send('Internal server error');
    });
    return function listen(port) {
        const server = http.createServer(app);
        server.on('listening', () => {
            utils_1.logger.info(`Express server is listening on port ${server.address().port}`);
        });
        server.on('error', (error) => {
            if (error.syscall !== 'listen') {
                throw error;
            }
            switch (error.code) {
                case 'EACCES':
                    utils_1.logger.error(`Port ${port} requires elevated privileges`);
                    process.exit(1);
                    break;
                case 'EADDRINUSE':
                    utils_1.logger.error(`Port ${port} is already in use`);
                    process.exit(1);
                    break;
                default:
                    throw error;
            }
        });
        server.listen(port);
    };
}
exports.default = generateServer;
//# sourceMappingURL=express.js.map